"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var graphql_codegen_core_1 = require("graphql-codegen-core");
var path = require("path");
// from https://github.com/npm/npm/blob/latest/lib/utils/parse-json.js
// from read-package-json
var stripBOM = function (content) {
    content = content.toString();
    // Remove byte order marker. This catches EF BB BF (the UTF-8 BOM)
    // because the buffer-to-string conversion in `fs.readFileSync()`
    // translates it to FEFF, the UTF-16 BOM.
    if (content.charCodeAt(0) === 0xfeff) {
        content = content.slice(1);
    }
    return content;
};
exports.parseBOM = function (content) { return JSON.parse(stripBOM(content)); };
exports.introspectionFromFile = function (file) {
    graphql_codegen_core_1.logger.info("Loading GraphQL Introspection from file: " + file + "...");
    return new Promise(function (resolve, reject) {
        var fullPath = path.isAbsolute(file) ? file : path.resolve(process.cwd(), file);
        if (fs.existsSync(fullPath)) {
            try {
                var fileContent = fs.readFileSync(fullPath, 'utf8');
                if (!fileContent) {
                    reject("Unable to read local introspection file: " + fullPath);
                }
                var introspection = exports.parseBOM(fileContent);
                if (introspection.data) {
                    introspection = introspection.data;
                }
                resolve(introspection);
            }
            catch (e) {
                reject(e);
            }
        }
        else {
            reject("Unable to locate local introspection file: " + fullPath);
        }
    });
};
//# sourceMappingURL=introspection-from-file.js.map